import java.util.List;
import java.util.Random;

/**
 * A simple model of a rabbit.
 * Rabbits age, move, breed, and die.
 * 
 * @author David J. Barnes and Michael Kolling
 * @version 2003.10.28
 */
public class Rabbit extends Animal
{
    // Characteristics shared by all rabbits (static fields).

    // The age at which a rabbit can start to breed.
    private static final int BREEDING_AGE = 5;
    // The age to which a rabbit can live.
    private static final int MAX_AGE = 50;
    // The likelihood of a rabbit breeding.
    private static final double BREEDING_PROBABILITY = 0.15;
    // The maximum number of births.
    private static final int MAX_LITTER_SIZE = 5;
    // A shared random number generator to control breeding.
    private static final Random rand = new Random();
    
    // Individual characteristics (instance fields).

    /**
     * Create a new rabbit. A rabbit may be created with age
     * zero (a new born) or with a random age.
     * 
     * @param randomAge If true, the rabbit will have a random age.
     */
    public Rabbit(boolean randomAge)
    {
        super();
        if(randomAge) {
            setAge(rand.nextInt(MAX_AGE));
        }
    }
    
    /** 
     * addBabies: moved out of act() by pld. 
     * You are to move this out of this class entirely, into Animal
     * Use the newAnimal() method instead of new Rabbit(false)
     */
    
    public void addBabies(Field updatedField, List newAnimals) {
            int births = breed();
            for(int b = 0; b < births; b++) {
                Rabbit newRabbit = new Rabbit(false);
                newAnimals.add(newRabbit);
                newRabbit.setLocation(
                        updatedField.randomAdjacentLocation(getLocation()));
                updatedField.place(newRabbit); 
            }
    }
    
    /**
     * This is what the rabbit does most of the time - it runs 
     * around. Sometimes it will breed or die of old age.
     * @param currentField The field currently occupied.
     * @param updatedField The field to transfer to.
     * @param newAnimals A list to add newly born rabbits to.
     */
    public void act(Field currentField, Field updatedField, List newAnimals)
    {
        incrementAge();
        incrementHunger();  // new -- pld: does nothing for Rabbit
        if(isAlive()) {
            addBabies(updatedField, newAnimals);
      
            Location newLocation;
            // newLocation = updatedField.freeAdjacentLocation(getLocation());
            // Only transfer to the updated field if there was a free location
            
            //Replace the following ONE line with the line below: -- pld
            newLocation = updatedField.freeAdjacentLocation(getLocation());
            // pld: newLocation = newLocation(currentField, updatedField);

            // the following stays part of act() -- pld
            if(newLocation != null) {
                setLocation(newLocation);
                updatedField.place(this);
            }
            else {
                // can neither move nor stay - overcrowding - all locations taken
                setDead();
            }

        }
    }
    
    /**
     * Increase the age.
     * This could result in the rabbit's death.
     *
    public void incrementAge()
    {
        setAge(getAge() + 1);
        if(getAge() > MAX_AGE) {
            setDead();
        }
    }
    /* */
    /**
     * Generate a number representing the number of births,
     * if it can breed.
     * @return The number of births (may be zero).
     * pld: you will move this to Animal, after replacing the constants with accessors
     */
    
    private int breed()
    {
        int births = 0;
        if(canBreed() && rand.nextDouble() <= BREEDING_PROBABILITY) {
            births = rand.nextInt(MAX_LITTER_SIZE) + 1;
        }
        return births;
    }
    
    /**
     * @return A string representation of the rabbit.
     */
    public String toString()
    {
        return "Rabbit, age " + getAge();
    }

    /**
     * A rabbit can breed if it has reached the breeding age.
     * pld: you will move this to Animal, after replacing the constants with accessors
     */
     private boolean canBreed()
    {
        return getAge() >= BREEDING_AGE;
    }
    
    public int getBreedingAge() {
        return BREEDING_AGE;
    }
    public int getMaxAge() {
        return MAX_AGE;
    }
    public double breedingProbability () {
        return BREEDING_PROBABILITY;
    }
    public int maxLitter() {
        return MAX_LITTER_SIZE;
    }
    
    public Animal newAnimal() {
        return new Rabbit(false);
    }

    // added by pld; to be called by the uniformized version of act()
    public Location newLocation(Field current, Field updated) {
            Location newLocation = updated.freeAdjacentLocation(getLocation());
            return newLocation;
    }

    // a dummy to allow uniformization of Rabbit (which doesn't need this in act())
    // and Fox (which does)
    public void incrementHunger() {
    }
}
